﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace HighVoltz.HBToggle
{
    public partial class FormHBToggle : Form
    {
    public FormHBToggle() { InitializeComponent(); }

        private bool _keyBindMode;

        private void KeyBindButtonClick(object sender, EventArgs e)
        {
            if (!_keyBindMode)
            {
                var btn = (Button)sender;
                btn.BackColor = SystemColors.GradientInactiveCaption;
                btn.FlatStyle = FlatStyle.Flat;
                btn.Text = "Press key combination";
                _keyBindMode = true;
            }
        }

        private void KeyBindButtonKeyDown(object sender, KeyEventArgs e)
        {
            // return if not changing keybinds.
            if (_keyBindMode)
            {
                var btn = (Button)sender;
                var key = GetKeyBind((Button)sender, e);
                if (key != Keys.None)
                {
                    switch (btn.Name)
                    {
                        case "_hbKeyBindButton":
                            HBToggleSettings.Instance.HBKeybind = key;
                            HBTogglePlugin.HBToggleKeybind.KeyBind = HBToggleSettings.Instance.HBKeybind;
                            break;
                        case "_movementKeyBindButton":
                            HBToggleSettings.Instance.MovementKeybind = key;
                            HBTogglePlugin.MovementKeybind.KeyBind = HBToggleSettings.Instance.MovementKeybind;
                            break;
                    }
                    HBToggleSettings.Instance.Save();
                    _keyBindMode = false;
                }
            }
        }

        private Keys GetKeyBind(Button sender, KeyEventArgs e)
        {
            // convert modifier keys to string to be displayed on button
            string keybindstring = (e.Shift ? "Shift " : "") + (e.Control ? "Ctrl " : "") + (e.Alt ? "Alt " : "");
            sender.Text = keybindstring;
            if (e.KeyCode == Keys.ShiftKey || e.KeyCode == Keys.ControlKey || e.KeyCode == Keys.Menu)
            {
                return Keys.None;
            }
            // display numbers in button text correctly.
            if (e.KeyCode >= Keys.D0 && e.KeyCode <= Keys.D9)
            {
                keybindstring += e.KeyValue - 48;
            }
            else
            {
                keybindstring += e.KeyCode.ToString();
            }
            sender.Text = keybindstring;
            sender.FlatStyle = FlatStyle.Standard;
            sender.BackColor = SystemColors.Control;
            return e.KeyData;
        }

        private void FormHBToggleShown(object sender, EventArgs e)
        {
            _hbKeyBindButton.Text = HBToggleSettings.Instance.HBKeybind == 0 ? "Click to set Keybind" : VkeyToString(HBToggleSettings.Instance.HBKeybind);
            _movementKeyBindButton.Text = HBToggleSettings.Instance.MovementKeybind == 0 ? "Click to set Keybind" : VkeyToString(HBToggleSettings.Instance.MovementKeybind);
        }

        private string VkeyToString(Keys vKey)
        {
            // extract modifier keys                 
            string keybindstring = ((vKey & Keys.Shift) != 0 ? "Shift " : "") +
                                   ((vKey & Keys.Control) != 0 ? "Ctrl " : "") + ((vKey & Keys.Alt) != 0 ? "Alt " : "");
            // Shift = 0x00010000, Ctrl= 0x00020000, Alt = 0x00040000
            // remove modifer bits
            vKey ^= (Keys.Shift & vKey) | (Keys.Control & vKey) | (Keys.Alt & vKey);
            // display numbers in button text correctly.
            switch (vKey)
            {
                case Keys.D0:
                    keybindstring += "0";
                    break;
                case Keys.D1:
                    keybindstring += "1";
                    break;
                case Keys.D2:
                    keybindstring += "2";
                    break;
                case Keys.D3:
                    keybindstring += "3";
                    break;
                case Keys.D4:
                    keybindstring += "4";
                    break;
                case Keys.D5:
                    keybindstring += "5";
                    break;
                case Keys.D6:
                    keybindstring += "6";
                    break;
                case Keys.D7:
                    keybindstring += "7";
                    break;
                case Keys.D8:
                    keybindstring += "8";
                    break;
                case Keys.D9:
                    keybindstring += "9";
                    break;
                default:
                    keybindstring += vKey.ToString();
                    break;
            }
            return keybindstring;
        }
    }
}
